/* ==========================================================[ CommandSet.hh ]=
 * CommandSet.hh - command callback functions
 * Copyright (C) 2002 Hubert Chan <hubert@uhoreg.ca>
 *
 * ----------------------------------------------------------------( License )-
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * ------------------------------------------------------------( Description )-
 *
 * This file maps command names to the functions implementing the commands.
 * Commands are attached to modes of the player, or can be accessible in all
 * modes.
 *
 * ------------------------------------------------------------------------- */

#ifndef __COMMAND_SET_HH_
#define __COMMAND_SET_HH_

#include <map>
#include <vector>
#include <string>
#include <stack>

#include <alsaplayer/CorePlayer.h>
#include <alsaplayer/Playlist.h>

#include "SessionData.hh"
#include "PlaylistInterface.hh"

class CommandSet : public ModeSubscriber
{
  void modeChangedCallback();

  typedef void (CommandSet::*callback_t)(const std::string &);

  // callbacks are indexed by an operating mode, then by the command name.
  static std::vector<std::map<std::string,callback_t> > callbacks;
  static bool is_initialized;

  // cached useful information
  std::map<std::string,callback_t> *curr_mode;
  static std::map<std::string,callback_t> &all_modes;

  float oldspeed; // keep track of the old speed for playing/pausing

  int selectedSong;

  playlistmode_t playlistMode;

  std::stack<int> selectedItemHistory;

  int selectedPref;

  // these are the callbacks.
  // general player callbacks:
  void play(const std::string &args);
  void stop(const std::string &args);
  void pause(const std::string &args);
  void playpause(const std::string &args);
  void next(const std::string &args);
  void prev(const std::string &args);
  void fwd(const std::string &args);
  void bwd(const std::string &args);
  void vol_up(const std::string &args);
  void vol_down(const std::string &args);
  void pan_left(const std::string &args);
  void pan_right(const std::string &args);
  void pan(const std::string &args);
  void faster(const std::string &args);
  void slower(const std::string &args);
  void speed(const std::string &args);
  // playlist-related callbacks
  void show_playlist(const std::string &args);
  void hide_playlist(const std::string &args);
  void playlist_up(const std::string &args);
  void playlist_down(const std::string &args);
  void playlist_next_mode(const std::string &args);
  void playlist_prev_mode(const std::string &args);
  void playlist_select(const std::string &args);
  // selector-related callbacks
  void exit_selector(const std::string &args);
  void selector_up(const std::string &args);
  void selector_down(const std::string &args);
  void selector_sublist(const std::string &args);
  void selector_backout(const std::string &args);
  void selector_select(const std::string &args);
  // menu-related callbacks
  void show_menu(const std::string &args);
  void exit_menu(const std::string &args);
  void menu_next_pref(const std::string &args);
  void menu_prev_pref(const std::string &args);
  void menu_next_value(const std::string &args);
  void menu_prev_value(const std::string &args);

  // initialize the callback table
  static void init (void);
public:
  CommandSet(SessionData &);

  void call(const std::string &command, const std::string &args);
};


#endif // __COMMAND_SET_HH_
