/* =========================================================[ TextDisplay.hh ]=
 * TextDisplay.hh - a text-based display
 * Copyright (C) 2002 Hubert Chan <hubert@uhoreg.ca>
 *
 * ----------------------------------------------------------------( License )-
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * ------------------------------------------------------------( Description )-
 *
 * This file implements a text-based display, such as XOSD, an external
 * character LCD or VFD display, or an ncurses display.
 *
 * ------------------------------------------------------------------------- */

#ifndef __TEXT_DISPLAY_HH_
#define __TEXT_DISPLAY_HH_

#include <string>
#include <vector>
#include <list>
#include <pthread.h>

#include "Display.hh"

class TextDisplay : public Display
{
public:
  class DisplayDriver
  {
    int width, height;
  public:
    DisplayDriver(int w, int h)
      : width(w), height(h) { }
    virtual ~DisplayDriver() { }

    int getWidth() const { return width; }
    int getHeight() const { return height; }
    virtual void print(unsigned int line, const std::string &str) = 0;
    virtual void hide() = 0;
    virtual void show() = 0;
  } *driver;

  class Window;
private:
  class PlayerWindow;
  class PopupWindow;
  class PlaylistWindow;
  class SelectorWindow;
  class PreferenceWindow;

  friend class TextDisplay::Window;

  pthread_mutex_t display_lock;

  PlayerWindow *playerWindow;
  PopupWindow *popupWindow;
  PlaylistWindow *playlistWindow;
  SelectorWindow *selectorWindow;
  PreferenceWindow *preferenceWindow;

  std::list<Window*> activeWindows;

public:
  TextDisplay(SessionData &);
  ~TextDisplay();

  int getWidth() const { return driver->getWidth(); }
  int getHeight() const { return driver->getHeight(); }

  // core player notification
  void volumeChanged(float vol);
  void speedChanged(float speed);
  void panChanged(float pan);
  void positionNotify(int frame);
  void playerStopped();
  void playerStarted();

  // player controls
  void playerPaused();
  void playerUnpaused();

  // playlist management
  void setCurrentSong(int pos);
  void setSelectedSong(int pos);
  void songsInserted(int pos);
  void songUpdated(int pos);
  void songsRemoved(int pos);
  void playlistCleared();
  void setPlaylistMode(playlistmode_t);

  // song selector
  void setSelection(int pos);
  void updateSelectionList();

  // preferences
  void setPreference(int pos);
  void updatePreference(int pos);

  // mode change
  void modeChangedCallback();
};

#endif //__TEXT_DISPLAY_HH_
